-- formats.e - Graphic formats reading/writing - HIP 2.0
-- Copyright (C) 2001  Davi Tassinari de Figueiredo
--
-- This program is distributed under the terms of the GNU General
-- Public License. Please read the documentation for more information.
--
-- This file has routines for identifying the file format for a picture
-- and calling the appropriate reader/writer for a specified format.
-- It may seem useless now, as only BMP files are supported, but
-- it is my intention to add support for other graphics formats
-- in future versions (GIF support is well on its way already, you may
-- even see some commented-out references to GIF in the code).

constant include_name = "formats.e"

include errors.e
include constant.e


include fileutil.e
include bmp.e

constant bytes_to_read = 4


global function get_pic_info (atom fn)
    object info
    sequence header

    -- log_info ("get_pic_info: start", LOG_IMPORTANT)
    -- Move to the beginning of the file
    if seek(fn, 0) then
	return ERROR_FILEERROR
    end if

    header = read_bytes (fn, bytes_to_read)
    -- log_info_s("get_pic_info: header = %02x %02x %02x %02x", header, LOG_IMPORTANT)
    -- unexpected eof?
    if length(header) < bytes_to_read then return ERROR_INVALIDBMP end if

    -- Bitmap ID - 2 bytes
    if compare (header[1..2], "BM") = 0 or
	  compare (header[1..2], "BA") = 0 then
	-- BMP file

	-- log_info ("get_pic_info: BMP identified, calling get_bmp_info", LOG_IMPORTANT)
	info = get_bmp_info (fn)

	if atom(info) then
	    -- log_info_s ("get_pic_info: get_bmp_info returned value %d",info, LOG_DETAIL)
	else
	    -- log_info_s ("get_pic_info: get_bmp_info returned a %d-element sequence",length(info), LOG_DETAIL)
	end if

	-- log_info ("get_pic_info: exiting with info", LOG_IMPORTANT)
	return info

    else
	-- log_info ("get_pic_info: no format identified, exiting with ERROR_INVALIDBMP", LOG_IMPORTANT)

	return ERROR_INVALIDBMP
    end if

end function



global function save_pic (atom format, atom fn, sequence bmp_info)

    if format != FORMAT_BMP then
	abort_error (include_name, "save_picture", "Unknown picture format")
    end if

    if format = FORMAT_BMP then
	return save_bmp (fn, bmp_info)

    end if

end function

