-- hip.ex - Hide In Picture 2.0 command-line interface
-- Copyright (C) 2001  Davi Tassinari de Figueiredo
--
-- This program is distributed under the terms of the GNU General
-- Public License. Please read the documentation for more information.
--
-- If you wish to contact me, send an e-mail to davitf@usa.net .
--
-- You can get the latest version of this program from my Euphoria page:
-- http://davitf.n3.net/
--
-- This file is the main program for the DOS/Linux version of HIP.


without warning
without type_check

include get.e
include wildcard.e
include graphics.e

wrap(0)

constant ESC = {27}


include formats.e
include messages.e

include main.e
include pictures.e
include fileutil.e

include sle.e

constant ERROR_ARG = 1,     -- Command-line error code

	-- Options sequence structure
	OPT_OPERATION = 1, OPT_FNTOWRITE = 2, OPT_ENCRYPTION = 3,
	OPT_PASSWORD = 4, OPT_WRITECRC = 5, OPT_HIDEPWD = 6,
	OPT_VIEWONLY = 7, OPT_QUIET = 8, OPT_TRANSPARENT = 9,
	OPT_FORMAT = 10, OPT_YES = 11,
	OPT_INPUTBMP = 12, OPT_FILE = 13, OPT_OUTPUTBMP = 14,

	-- Operation code
	OP_HIDE = 1, OP_RETRIEVE = 2, OP_ERASE = 3,

	ENC_DEFAULT = 1,
	-- Default options
	default_options = {-1, -1, 0, -1, 1, 1, 0, 0, -1, FORMAT_BMP, 0, -1, -1, -1}

constant hex_values = "0123456789abcdef"

atom show_messages, auto_yes
sequence screen_properties
screen_properties = video_config()


procedure show_help ()
    -- Shows the help screen, creating a list of the available
    -- encryption algorithms
    sequence algorithms
    algorithms = {}
    for n = 1 to length (enc_algs) do
	if ENC_DEFAULT = n then
	    algorithms = algorithms & '*'
	end if

	algorithms = algorithms & sprintf ("%s=%s", {'a' + n - 1, enc_algs [n][ALG_NAME]})
	if n < length (enc_algs) then algorithms = algorithms & "  " end if
    end for

    printf (1, help, {algorithms})
end procedure


constant YES = 1, NO = 2

function yes_or_no (sequence question, atom default)
    -- Asks a question and gets an answer from the user;
    -- if the key pressed is invalid, return default value.
    atom key, answer

    -- If the 'automatically answer yes' options is enabled,
    -- do not ask the user
    if auto_yes = 1 and show_messages = 0 then
	return YES

    elsif auto_yes = 1 then
	puts(1, question)
	answer = YES

    else        -- Ask the user
	puts(1, question)

	key = upper (wait_key())        -- Get key
	if key = yes_key then answer = YES
	elsif key = no_key then answer = NO
	else answer = default end if
    end if

    if answer = YES then puts (1, yes_key & '\n')
    elsif answer = NO then puts (1, no_key & '\n')
    else abort_error ("hip.ex", "yes_or_no", "Not YES or NO answer")
    end if

    return answer
end function


function get_color (sequence string)
    -- Converts a command-line color index into an atom or sequence
    atom val1, val2, index
    sequence rgb

    string = lower (string)
    if length(string) = 2 then
	val1 = find (string[1], hex_values) - 1
	val2 = find (string[2], hex_values) - 1
	if val1 = -1 or val2 = -1 then return -1 end if
	index = val1 * #10 + val2
	return index

    elsif length(string) = 6 then

	rgb = {-1, -1, -1}

	-- Red
	val1 = find (string[1], hex_values) - 1
	val2 = find (string[2], hex_values) - 1
	if val1 = -1 or val2 = -1 then return -1 end if
	rgb [PAL_RED] = val1 * #10 + val2

	-- Green
	val1 = find (string[3], hex_values) - 1
	val2 = find (string[4], hex_values) - 1
	if val1 = -1 or val2 = -1 then return -1 end if
	rgb [PAL_GREEN] = val1 * #10 + val2

	-- Blue
	val1 = find (string[5], hex_values) - 1
	val2 = find (string[6], hex_values) - 1
	if val1 = -1 or val2 = -1 then return -1 end if
	rgb [PAL_BLUE] = val1 * #10 + val2

	return rgb

    else
	return -1
    end if


end function




function get_command ()
    -- Get command-line arguments

    atom command, pos
    sequence comm_line, file_names, options, arg
    object transparent

    options = default_options

    comm_line = command_line()


    comm_line = comm_line [3..length(comm_line)]
    if length(comm_line) < 2 then return ERROR_ARG end if

    command = comm_line[1][1]

    -- Get operation
    if command = 'h' then
	options [OPT_OPERATION] = OP_HIDE
    elsif command = 'r' then
	options [OPT_OPERATION] = OP_RETRIEVE
    elsif command = 'e' then
	options [OPT_OPERATION] = OP_ERASE
    else
	return ERROR_ARG
    end if


    if length(comm_line[1]) >= 2 then
	if comm_line[1][2] = '-' then   -- Options
	    comm_line [1] = comm_line [1][2..length(comm_line[1])]
	else return ERROR_ARG end if    -- Error

    else
	comm_line = comm_line [2..length(comm_line)]
    end if

    file_names = {}

    while length(comm_line) do

	arg = comm_line [1]
	comm_line = comm_line [2..length(comm_line)]


	if arg[1] != '-' then
	    file_names = append (file_names, arg)

	else
	    arg = arg [2..length(arg)]


	    if compare (arg[1], 'f') = 0 then
		-- File name to write
		options [OPT_FNTOWRITE] = arg [2..length(arg)]

	    elsif compare (arg[1], 'p') = 0 then
		options [OPT_PASSWORD] = arg [2..length(arg)]

	    elsif compare (arg[1], 't') = 0 then
		transparent = get_color (arg [2..length(arg)])
		if compare (transparent, -1) = 0 then
		    -- Error
		    return ERROR_ARG
		end if

		options [OPT_TRANSPARENT] = transparent

	    else

		pos = find ('-', arg)
		if pos then
		    comm_line = prepend (comm_line, arg [pos..length(arg)])
		    arg = arg [1..pos-1]
		end if

		if length (arg) = 2 and arg[1] = 'e' and
		    arg[2] >= 'a' and arg[2] <= 'a' + length (enc_algs)-1 then
		       options [OPT_ENCRYPTION] = arg[2] - 'a' + 1

		elsif compare (arg, "c") = 0 then
		    options [OPT_WRITECRC] = 1
		elsif compare (arg, "C") = 0 then
		    options [OPT_WRITECRC] = 0

		elsif compare (arg, "h") = 0 then
		    options [OPT_HIDEPWD] = 1
		elsif compare (arg, "H") = 0 then
		    options [OPT_HIDEPWD] = 0

--                 elsif compare (arg, "b") = 0 then
--                     options [OPT_FORMAT] = FORMAT_BMP
--                 elsif compare (arg, "g") = 0 then
--                     options [OPT_FORMAT] = FORMAT_GIF

		elsif compare (arg, "v") = 0 then
		    options [OPT_VIEWONLY] = 1

		elsif compare (arg, "q") = 0 then
		    options [OPT_QUIET] = 1
		elsif compare(arg, "y") = 0 then
		    options [OPT_YES] = 1
		else
		    return ERROR_ARG
		end if

	    end if

	end if

    end while


    if options [OPT_OPERATION] = OP_HIDE then
	if length(file_names) = 2 then
	    options [OPT_INPUTBMP] = file_names [1]
	    options [OPT_FILE] = file_names [2]
	    options [OPT_OUTPUTBMP] = file_names [1]

	elsif length(file_names) = 3 then
	    options [OPT_INPUTBMP] = file_names [1]
	    options [OPT_FILE] = file_names [2]
	    options [OPT_OUTPUTBMP] = file_names [3]
	else
	    return ERROR_ARG
	end if

	if atom (options [OPT_FNTOWRITE]) then
	    options [OPT_FNTOWRITE] = get_name (options [OPT_FILE])
	end if

    elsif options [OPT_OPERATION] = OP_RETRIEVE then

	if length(file_names) = 1 then
	    options [OPT_INPUTBMP] = file_names [1]
	else
	    options [OPT_INPUTBMP] = file_names [1]
	    options [OPT_FILE] = file_names [2]
	end if

    elsif options [OPT_OPERATION] = OP_ERASE then
	if length(file_names) = 1 then
	    options [OPT_INPUTBMP] = file_names [1]
	    options [OPT_OUTPUTBMP] = file_names [1]

	elsif length(file_names) = 2 then
	    options [OPT_INPUTBMP] = file_names [1]
	    options [OPT_OUTPUTBMP] = file_names [2]
	else
	    return ERROR_ARG
	end if


    end if

--     if options [OPT_OPERATION] = OP_HIDE and options [OPT_FORMAT] = -1 then
--
--         -- set default format
--         if length (options [OPT_OUTPUTBMP]) >= 4 and compare (upper(options [OPT_OUTPUTBMP]
--         [length (options [OPT_OUTPUTBMP])-3..length (options [OPT_OUTPUTBMP])]),
--         ".GIF") = 0 then
--             -- extension is GIF
--             options [OPT_FORMAT] = FORMAT_GIF
--         else
--             options [OPT_FORMAT] = FORMAT_BMP
--         end if
--
--     end if

    return options
end function

procedure show_information (sequence info, atom always)
    if always or show_messages then
	puts(1, info)
    end if
end procedure

function ask_password (atom confirmation, atom hide)
    atom char
    sequence cursor_pos
    object password, confirm


    if hide then char = '*' else char = -1 end if

    while 1 do

	puts(1, prompt_enterpassword)
	cursor_pos = get_position()
	password = ask (cursor_pos[1], cursor_pos[2], screen_properties [VC_COLUMNS],
		    "", char)

	puts(1, "\n")
	if atom (password) then return -1 end if

	if confirmation then
	puts (1, prompt_confirmpassword)
	    cursor_pos = get_position()
	    confirm = ask (cursor_pos[1], cursor_pos[2],
		      screen_properties [VC_COLUMNS], "", char)
	    puts(1, "\n")

	    if atom (confirm) then return -1
	    elsif compare (password, confirm) = 0 then
		return password end if

	else return password    -- No confirmation needed
	end if

	puts(1, error_differentpasswords & '\n')

    end while


end function

constant highlighted_color = BRIGHT_WHITE, normal_color = BLUE,
	 highlighted_char = '*', normal_char = '*',
	 txt_color = WHITE, back_color = BLACK

procedure show_status (atom line, atom col, atom width, atom status)
    atom highlighted

    if show_messages = 0 then return end if     -- Quiet mode

    width = width - 5

    highlighted = floor (status * width)

    bk_color (back_color)

    position (line, col)
    text_color (highlighted_color)
    puts (1, repeat (highlighted_char, highlighted))

    text_color (normal_color)
    puts (1, repeat (normal_char, width - highlighted))

    text_color (txt_color)
    printf (1, " %3d%%", status*100)

end procedure

constant status_interval = .05

procedure hide (sequence options)
    atom input_bmp, output_bmp, input_fn,
	 error, timer, bar_width
    object bmp_info
    sequence pos

    -- Confirm overwrite

    if options [OPT_VIEWONLY] = 0 then

	if compare (options [OPT_INPUTBMP], options [OPT_OUTPUTBMP]) = 0 then
	    if yes_or_no (sprintf (prompt_overwritepicture, {get_name(options [OPT_INPUTBMP]) }),
	      YES) = NO then end_program (error_abortedbyuser) end if

	elsif file_exists (options [OPT_OUTPUTBMP]) then
	    if yes_or_no (sprintf(prompt_overwrite, { get_name (options [OPT_OUTPUTBMP] )}),
	      NO) = NO then end_program (error_abortedbyuser) end if

	end if

    end if


    -- Open input file
    input_fn = open (options [OPT_FILE], "rb")
    if input_fn = -1 then  -- Could not open file
	end_program(sprintf (error_unabletoopen, {options [OPT_FILE]}))
    end if


    show_information (sprintf (status_opening_bmp & '\n', {get_name(options [OPT_INPUTBMP])}), 0)
    -- Open input bitmap
    input_bmp = open (options [OPT_INPUTBMP], "rb")
    if input_bmp = -1 then  -- Could not open file
	end_program(sprintf (error_unabletoopen, {options [OPT_INPUTBMP]}))
    end if



    -- Read bitmap information
    bmp_info = get_pic_info (input_bmp)

    if atom (bmp_info) then     -- Error in bitmap
	close (input_bmp)
	if bmp_info = ERROR_FILEERROR then
	end_program(sprintf (error_filesystemerror, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_CORRUPTED then
	end_program(sprintf (error_corrupted, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_UNSUPPORTED then
	end_program(sprintf (error_unsupportedbmp, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_INVALIDBMP then
	end_program(sprintf (error_invalidbmp, {options [OPT_INPUTBMP]}))

	else
	end_program(sprintf (error_unknown, {options [OPT_INPUTBMP], bmp_info}))
	end if

    end if

    -- No errors, bitmap loaded successfully
    close (input_bmp)


    -- See whether output format supports BPP
--     if bmp_info [BMP_BPP] != 8 and options [OPT_FORMAT] = FORMAT_GIF then
--         -- error
--         end_program (error_rgbnotgif)
--     end if



    -- Set transparent color
    if compare(options [OPT_TRANSPARENT] , -1) then
	if bmp_info [BMP_BPP] = 8 and sequence (options [OPT_TRANSPARENT]) then
		options [OPT_TRANSPARENT] =
		find (options [OPT_TRANSPARENT], bmp_info [BMP_PAL]) - 1
		if options [OPT_TRANSPARENT] = -1 then
		    end_program (error_colornotfound)
		end if
	elsif bmp_info [BMP_BPP] = 24 and atom (options [OPT_TRANSPARENT]) then
	    end_program (error_mustbergb)
	end if

	bmp_info = set_transparent_color (bmp_info, options [OPT_TRANSPARENT])

    end if

    -- Is picture big enough to store the whole file?
    if total_bits (file_size (input_fn), 0,
       options [OPT_WRITECRC], length (options [OPT_FNTOWRITE]), 0) >
       bmp_info [BMP_MAXBITSPERBYTE] * bmp_info [BMP_AVAILABLEBYTES]

       then close (input_fn) end_program (error_filetoobig)

    end if

    if options [OPT_VIEWONLY] then      -- Do not really write, just check space
	close (input_fn)
	puts(1, info_filecanbehidden & '\n')
	return
    end if

    -- No problems
    -- Is password already set?
    if atom (options [OPT_PASSWORD]) then
	options [OPT_PASSWORD] = ask_password (1, options [OPT_HIDEPWD])
	if atom(options [OPT_PASSWORD]) then    -- User exited password prompt
	    end_program (error_abortedbyuser) end if

    end if

    if atom (options [OPT_FNTOWRITE]) then
	options [OPT_FNTOWRITE] = options [OPT_FILE] end if

    if options [OPT_ENCRYPTION] = 0 then
	options [OPT_ENCRYPTION] = ENC_DEFAULT
    end if

    show_information (status_initializing & '\n', 0)

    error = init_writing (bmp_info [BMP_AVAILABLEBYTES],
	    input_fn,
	    options [OPT_WRITECRC],
	    options [OPT_FNTOWRITE],
	    {},
	    options [OPT_ENCRYPTION],
	    options [OPT_PASSWORD],
	    bmp_info [BMP_MAXBITSPERBYTE] * bmp_info [BMP_AVAILABLEBYTES])

    if error = ERROR_FILETOOBIG then
	-- This test has already been done, this should not happen
	abort_error ("hip.ex", "hide", "init_writing reports file_too_big")

    elsif error = ERROR_FILEERROR then
	end_program(sprintf (error_filesystemerror, {options [OPT_FILE]}))
    end if


    -- Now it's the fun part - let's hide the file!

    show_information (status_hiding, 0)

    pos = get_position()
    timer = time() + status_interval
    bar_width = screen_properties [VC_COLUMNS] - pos [2] - 1

    error = 0
    while error = 0 do
	if time() >= timer then
	    show_status (pos[1], pos[2], bar_width, completion_status () )
	    timer = time() + status_interval
	    if find (get_key() , ESC) then
		show_information ("\n", 0)
		end_program (error_abortedbyuser)
	    end if
	end if
	error = go_write()
    end while

    close (input_fn)

    if error = ERROR_FILEERROR then
	end_program(sprintf (error_filesystemerror, {options [OPT_FILE]}))
    elsif error != INFO_LASTBLOCK then
	abort_error ("hip.ex", "hide", sprintf("Unknown error code from go_write: %d", error))
    end if

    -- Done, now write the hidden file in the picture

    show_status (pos[1], pos[2], bar_width, 1)

    show_information ("\n", 0)
    show_information (status_writing_in_picture, 0)

    pos = get_position()
    timer = time() + status_interval
    bar_width = screen_properties [VC_COLUMNS] - pos [2] - 1

    init_write_data_in_bmp ( get_scattered_data (), get_current_bit(),
		bmp_info)


    end_scatter ()

    error = 0

    while error = 0 do
	if time() >= timer then
	    show_status (pos[1], pos[2], bar_width, bmp_completion_status () )
	    timer = time() + status_interval
	    if find (get_key() , ESC) then
		show_information ("\n", 0)
		end_program (error_abortedbyuser)
	    end if
	end if
	error = go_write_in_bmp()
    end while

    if error != INFO_LASTBLOCK then
	abort_error ("hip.ex", "hide", sprintf("Unknown error code from go_write_in_bmp: %d", error))
    end if

    show_status (pos[1], pos[2], bar_width, 1)

    -- Done writing file in picture, now save picture

    show_information ("\n", 0)
    show_information (sprintf(status_saving_bmp & '\n', {get_name(options [OPT_OUTPUTBMP])} ), 0)

    output_bmp = open (options [OPT_OUTPUTBMP], "wb")

    if output_bmp = -1 then
	end_program (sprintf (error_unabletoopen, {options [OPT_OUTPUTBMP]}))
    end if


    if save_pic (options [OPT_FORMAT], output_bmp, bmp_info) then
	abort_error ("hip.ex", "hide", "Error in save picture routine")
    end if

    close (output_bmp)

    show_information ("\n", 0)
    show_information (status_done & '\n', 0)
    -- Done!
end procedure


procedure retrieve (sequence options)
    atom input_bmp, output_fn,
	 error, timer, bar_width
    object bmp_info, hidden_info
    sequence pos, data


    show_information (sprintf (status_opening_bmp & '\n', {get_name(options [OPT_INPUTBMP])}),0)
    -- Open input bitmap
    input_bmp = open (options [OPT_INPUTBMP], "rb")
    if input_bmp = -1 then  -- Could not open file
	end_program(sprintf (error_unabletoopen, {options [OPT_INPUTBMP]}))
    end if


    -- Read bitmap information
    bmp_info = get_pic_info (input_bmp)

    if atom (bmp_info) then     -- Error in bitmap
	close (input_bmp)
	if bmp_info = ERROR_FILEERROR then
	end_program(sprintf (error_filesystemerror, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_CORRUPTED then
	end_program(sprintf (error_corrupted, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_UNSUPPORTED then
	end_program(sprintf (error_unsupportedbmp, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_INVALIDBMP then
	end_program(sprintf (error_invalidbmp, {options [OPT_INPUTBMP]}))

	else
	end_program(sprintf (error_unknown, {options [OPT_INPUTBMP], bmp_info}))
	end if

    end if


    -- No errors, bitmap loaded successfully
    close (input_bmp)


    -- Set transparent color
    if compare(options [OPT_TRANSPARENT] , -1) then

	if bmp_info [BMP_BPP] = 8 and sequence (options [OPT_TRANSPARENT]) then
		options [OPT_TRANSPARENT] =
		find (options [OPT_TRANSPARENT], bmp_info [BMP_PAL]) - 1
		if options [OPT_TRANSPARENT] = -1 then
		    end_program (error_colornotfound)
		end if
	elsif bmp_info [BMP_BPP] = 24 and atom (options [OPT_TRANSPARENT]) then
	    end_program (error_mustbergb)
	end if

	bmp_info = set_transparent_color (bmp_info, options [OPT_TRANSPARENT])

    end if



    -- Is password already set?
    if atom (options [OPT_PASSWORD]) then
	options [OPT_PASSWORD] = ask_password (0, options [OPT_HIDEPWD])
	if atom(options [OPT_PASSWORD]) then    -- User exited password prompt
	    end_program (error_abortedbyuser) end if

    end if



    show_information (status_reading_from_picture, 0)

    pos = get_position()
    timer = time() + status_interval
    bar_width = screen_properties [VC_COLUMNS] - pos [2] - 1

    init_read_data_from_bmp (bmp_info)



    error = 0

    while error = 0 do
	if time() >= timer then
	    show_status (pos[1], pos[2], bar_width, bmp_completion_status () )
	    timer = time() + status_interval
	    if find (get_key() , ESC) then
		show_information ("\n", 0)
		end_program (error_abortedbyuser)
	    end if
	end if
	error = go_read_from_bmp()
    end while

    if error != INFO_LASTBLOCK then
	abort_error ("hip.ex", "retrieve", sprintf("Unknown error code from go_read_in_bmp: %d", error))
    end if

    show_status (pos[1], pos[2], bar_width, 1)

    show_information ("\n", 0)

    show_information( status_initializing & '\n', 0)

    init_scatter_read (get_bmp_data () )
    end_bmp ()

    hidden_info = get_hidden_info (options [OPT_ENCRYPTION],
		  options [OPT_PASSWORD],
	    bmp_info [BMP_MAXBITSPERBYTE] * bmp_info [BMP_AVAILABLEBYTES] )

    if atom (hidden_info) then
	if hidden_info = ERROR_NOHIDDENFILE then
	    end_program (error_nohiddenfile)
	elsif hidden_info = ERROR_UNSUPPORTED then
	    end_program (error_unsupported)
	elsif hidden_info = ERROR_CORRUPTED then
	    end_program (error_hiddencorrupted)
	else
	    abort_error ("hip.ex", "retrieve", sprintf("Unknown error code from get_hidden_info: %d", hidden_info))
	end if

    end if

    show_information (sprintf(info_filesize & '\n', hidden_info [2]), 0)


    if length( hidden_info [1]) = 0 then
	show_information (info_nofilename & '\n', 0)
	if atom (options [OPT_FILE]) and options [OPT_VIEWONLY] = 0 then
	    end_program (error_filenameneeded & '\n')
	end if
    else
	show_information (sprintf(info_filename & '\n',  {hidden_info [1]}),0)
	if atom (options [OPT_FILE]) then
	    options [OPT_FILE] = hidden_info [1]
	end if
    end if

    if options [OPT_VIEWONLY] then
	show_information (sprintf(info_algorithmused & '\n', {enc_algs [hidden_info[3]][ALG_NAME]}),0)

	if hidden_info [4] then
	    show_information (info_crcpresent & '\n', 0)
	else
	    show_information (info_crcnotpresent & '\n', 0)
	end if

	return
    end if


    -- Confirm overwrite
    if file_exists (options [OPT_FILE]) then
	if yes_or_no (sprintf(prompt_overwrite, { get_name (options [OPT_FILE] )}),
	NO) = NO then end_program (error_abortedbyuser) end if
    end if

    -- Open file
    output_fn = open (options [OPT_FILE], "wb")

    if output_fn = -1 then  -- Could not open file
	end_program(sprintf (error_unabletoopen, {options [OPT_FILE]}))
    end if

    init_reading (output_fn)

    show_information (status_retrieving, 0)


    -- Now it's the fun part - let's retrieve the file!

    pos = get_position()
    timer = time() + status_interval
    bar_width = screen_properties [VC_COLUMNS] - pos [2] - 1

    error = 0
    while error = 0 do
	if time() >= timer then
	    show_status (pos[1], pos[2], bar_width, completion_status () )
	    timer = time() + status_interval
	    if find (get_key() , ESC) then
		show_information ("\n", 0)
		end_program (error_abortedbyuser)
	    end if
	end if
	error = go_read()
    end while

    close (output_fn)

    show_status (pos[1], pos[2], bar_width, 1)

    show_information ("\n", 0)

    if error = ERROR_CORRUPTED then
	end_program(error_hiddencorrupted)
    elsif error != INFO_LASTBLOCK then
	abort_error ("hip.ex", "retrieve", sprintf("Unknown error code from go_read: %d", error))
    end if

    show_information (status_done & '\n', 0)
    -- Done!
end procedure

procedure erase (sequence options)
    atom input_bmp, output_bmp, input_fn,
	 error, timer, bar_width
    object bmp_info
    sequence pos, erase_data

    -- Confirm overwrite
    if options [OPT_VIEWONLY] then
	show_information (info_nothingtodo, 1)
	return
    end if

    if compare (options [OPT_INPUTBMP], options [OPT_OUTPUTBMP]) = 0 then
	if yes_or_no (sprintf (prompt_overwritepicture, {get_name(options [OPT_INPUTBMP]) }),
	  YES) = NO then end_program (error_abortedbyuser) end if

    elsif file_exists (options [OPT_OUTPUTBMP]) then
	if yes_or_no (sprintf(prompt_overwrite, { get_name (options [OPT_OUTPUTBMP] )}),
	  NO) = NO then end_program (error_abortedbyuser) end if

    end if

    show_information (sprintf (status_opening_bmp & '\n', {get_name(options [OPT_INPUTBMP])}), 0)
    -- Open input bitmap
    input_bmp = open (options [OPT_INPUTBMP], "rb")
    if input_bmp = -1 then  -- Could not open file
	end_program(sprintf (error_unabletoopen, {options [OPT_INPUTBMP]}))
    end if



    -- Read bitmap information
    bmp_info = get_pic_info (input_bmp)

    if atom (bmp_info) then     -- Error in bitmap
	close (input_bmp)
	if bmp_info = ERROR_FILEERROR then
	end_program(sprintf (error_filesystemerror, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_CORRUPTED then
	end_program(sprintf (error_corrupted, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_UNSUPPORTED then
	end_program(sprintf (error_unsupportedbmp, {options [OPT_INPUTBMP]}))

	elsif bmp_info = ERROR_INVALIDBMP then
	end_program(sprintf (error_invalidbmp, {options [OPT_INPUTBMP]}))

	else
	end_program(sprintf (error_unknown, {options [OPT_INPUTBMP], bmp_info}))
	end if

    end if

    -- No errors, bitmap loaded successfully
    close (input_bmp)

    -- Set transparent color
    if compare(options [OPT_TRANSPARENT] , -1) then
	if bmp_info [BMP_BPP] = 8 and sequence (options [OPT_TRANSPARENT]) then
		options [OPT_TRANSPARENT] =
		find (options [OPT_TRANSPARENT], bmp_info [BMP_PAL]) - 1
		if options [OPT_TRANSPARENT] = -1 then
		    end_program (error_colornotfound)
		end if
	elsif bmp_info [BMP_BPP] = 24 and atom (options [OPT_TRANSPARENT]) then
	    end_program (error_mustbergb)
	end if

	bmp_info = set_transparent_color (bmp_info, options [OPT_TRANSPARENT])

    end if

    -- No problems

    show_information (status_initializing & '\n', 0)

    -- Get erase data
    erase_data = get_erase_data (bmp_info [BMP_AVAILABLEBYTES],
	    options [OPT_PASSWORD])

    show_information ("\n", 0)
    show_information (status_writing_in_picture, 0)

    -- Init writing in picture
    pos = get_position()

    timer = time() + status_interval
    bar_width = screen_properties [VC_COLUMNS] - pos [2] - 1

    init_write_data_in_bmp ( erase_data, 1, bmp_info)

    erase_data = {}
    error = 0

    while error = 0 do
	if time() >= timer then
	    show_status (pos[1], pos[2], bar_width, bmp_completion_status () )
	    timer = time() + status_interval
	    if find (get_key() , ESC) then
		show_information ("\n", 0)
		end_program (error_abortedbyuser)
	    end if
	end if
	error = go_write_in_bmp()
    end while

    if error != INFO_LASTBLOCK then
	abort_error ("hip.ex", "erase", sprintf("Unknown error code from go_write_in_bmp: %d", error))
    end if

    show_status (pos[1], pos[2], bar_width, 1)

    -- Done writing data in picture, now save picture

    show_information ("\n", 0)
    show_information (sprintf(status_saving_bmp & '\n', {get_name(options [OPT_OUTPUTBMP])} ), 0)

    output_bmp = open (options [OPT_OUTPUTBMP], "wb")

    if output_bmp = -1 then
	end_program (sprintf (error_unabletoopen, {options [OPT_OUTPUTBMP]}))
    end if

    if save_pic (options [OPT_FORMAT], output_bmp, bmp_info) then
	abort_error ("hip.ex", "hide", "Error in save picture routine")
    end if

    close (output_bmp)

    show_information ("\n", 0)
    show_information (status_done & '\n', 0)
    -- Done!

end procedure



object x

x = get_command ()

if atom(x) then show_help() abort(0) end if -- Error in parsing command line

auto_yes = x [OPT_YES]      -- Automatically answer yes?
show_messages = not (x [OPT_QUIET])     -- Show messages?

show_information(program_id & "\n\n", 0)

if x [OPT_OPERATION] = OP_HIDE then
    hide (x)
elsif x [OPT_OPERATION] = OP_RETRIEVE then
    retrieve (x)
elsif x [OPT_OPERATION] = OP_ERASE then
    erase (x)
else    -- Bug, this should not happen
    abort_error ("hip.ex", "none", sprintf("Unknown operation code: %d", x[OPT_OPERATION]))
end if

abort(0)

